-----------------------------------------------------------------------------------------------
-- Client Lua Script for TradeskillTracker
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "CraftingLib"
require "GameLib"
require "AchievementsLib"
require "ChatSystemLib"
 
-----------------------------------------------------------------------------------------------
-- TradeskillTracker Module Definition
-----------------------------------------------------------------------------------------------
local TradeskillTracker = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local kcrSelectedText = ApolloColor.new("UI_BtnTextHoloPressedFlyby")
local kcrNormalText = ApolloColor.new("UI_BtnTextHoloNormal")
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function TradeskillTracker:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.tItems = {} -- keep track of all the list items
	o.wndSelectedListItem = nil -- keep track of which list item is currently selected

    return o
end

function TradeskillTracker:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {				-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function TradeskillTracker:OnSave(eLevel)
	-- We save at a character level in case of alts in different guilds
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return
	end

	tSave = {}
	local tMetaData = tMetaData
	local tTradeskills = tTradeskillChapter
	local tRecipeBook = tRecipeBook
	--local tTradeskillDiary = tTradeskillDiary	
	--local tSchematicAppendix = tTradeskillAppendix

	
	
		

	
	--local tSavedExportValues = {nTimeLastExported,strGuild,tHeaders,tRosterData}

	--if tTradeskillDiary ~= nil then
	--	tSave["tRecipes"] = tTradeskillDiary
	--end
	if tMetaData ~= nil then
		tSave["MetaData"] = tMetaData
	end
	
	if tTradeskills ~= nil then
		tSave["Trades"] = tTradeskills
	end
	
	--if tSchematicAppendix ~= nil then
	--	tSave["tSchematicList"] = tSchematicAppendix
	--end
	if tRecipeBook then
		tSave["TradeskillRecipes"] = tRecipeBook
	end
	
	return tSave
end

function TradeskillTracker:OnRestore(eLevel,tSaveData)
	if tSaveData.MetaData then 
		if tSaveData.MetaData.ProfileId then
			self.ProfileId = tSaveData.MetaData.ProfileId
		end
	end
end
	
	
 

-----------------------------------------------------------------------------------------------
-- TradeskillTracker OnLoad
-----------------------------------------------------------------------------------------------
function TradeskillTracker:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("TradeskillTracker.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
end

-----------------------------------------------------------------------------------------------
-- TradeskillTracker OnDocLoaded
-----------------------------------------------------------------------------------------------
function TradeskillTracker:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "TradeskillTrackerForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		-- item list
		self.wndItemList = self.wndMain:FindChild("ItemList")
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterEventHandler("WindowManagementReady", 			"OnWindowManagementReady", self)
		Apollo.RegisterSlashCommand("tt", "OnTradeskillTrackerOn", self)
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
		self.timerDelay = ApolloTimer.Create(1, false, "OnDelayTimer", self)		
		-- Do additional Addon initialization here

		ChatSystemLib.PostOnChannel(2,"TradeskillTracker: Tradeskill Tracker loaded and updating.")	
		
			
	end

	
end

-----------------------------------------------------------------------------------------------
-- TradeskillTracker Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function TradeskillTracker:OnDelayTimer()
	if GameLib.GetPlayerUnit() then 
		self.timerDelay = nil 
		self:OnInit()
	
	else self.timerDelay:Start()
	end
end

function TradeskillTracker:OnCharacterCreated()
	self:OnInit()
end
-- on SlashCommand "/tt"
function TradeskillTracker:OnTradeskillTrackerOn()
	self.wndMain:Invoke() -- show the window
	
	end
	
function TradeskillTracker:OnInit()
	if self.ProfileId ~= nil then
		self.wndMain:FindChild("strProfileId"):SetText(self.ProfileId)
	else 
		local ProfileId = self:GenerateKey()
		self.wndMain:FindChild("strProfileId"):SetText(ProfileId)
	end
		
	--Prebuild list
	local tTradeskillsList = { }
	for idx, tCurrTradeskill in ipairs(CraftingLib.GetKnownTradeskills()) do
		local tCurrTradeskillInfo = CraftingLib.GetTradeskillInfo(tCurrTradeskill.eId)
		if not tCurrTradeskillInfo.bIsHarvesting and idx ~= CraftingLib.CodeEnumTradeskill.Runecrafting then
			table.insert(tTradeskillsList, { tCurrTradeskill, tCurrTradeskillInfo })
		end
	end
	tTradeskillChapter = self:BuildKnownTradeskills(tTradeskillsList)
	tTradeskillAppendix = CraftingLib.GetSchematicList()
		
	local tKnownSchematicsList = { }
	for idx, tCurrTradeskill in ipairs(CraftingLib.GetKnownTradeskills()) do
		if idx ~= CraftingLib.CodeEnumTradeskill.Runecrafting then
			local tCurrTradeskillInfo = CraftingLib.GetTradeskillInfo(tCurrTradeskill.eId)
			for idx2, tSchematic in pairs(CraftingLib.GetSchematicList(tCurrTradeskill.eId, nil, nil, true)) do
				if tCurrTradeskillInfo.bIsHobby or tCurrTradeskillInfo.eTier >= tSchematic.eTier then
					if tSchematic.bIsKnown == true then
						local tSchematicInfo = CraftingLib.GetSchematicInfo(tSchematic.nSchematicId)
						table.insert(tKnownSchematicsList, {tSchematic,tSchematicInfo})
												
					end
					if tSchematic.bIsKnown == false then
						--following 2 lines used with bIsKnown == true to populate full schematic list for known professions
						--local tSchematicInfo = CraftingLib.GetSchematicInfo(tSchematic.nSchematicId)
						--table.insert(tKnownSchematicsList, {tSchematic,tSchematicInfo})
										
					end
				end
			end
		end
	end	
	local nRecipesKnown = #tKnownSchematicsList
	tRecipeChapter = self:BuildKnownSkillBook(tKnownSchematicsList)
	
	
	tTradeskillDiary = tRecipeChapter --self:BuildTradeskillDiary(tTradeskillChapter,tRecipeChapter)
	tMetaData = self:BuildTradeskillMetaData(nRecipesKnown)
	-- populate the item list
	tRecipeBook = self:BuildSortedTradeskillContent(tTradeskillChapter,tRecipeChapter)
	self:PopulateItemList(tTradeskillChapter,tRecipeChapter)
end

function TradeskillTracker:OnTradeskills_Learned()
-- Prebuild list
	local tTradeskillsList = { }
	for idx, tCurrTradeskill in ipairs(CraftingLib.GetKnownTradeskills()) do
		local tCurrTradeskillInfo = CraftingLib.GetTradeskillInfo(tCurrTradeskill.eId)
		if not tCurrTradeskillInfo.bIsHarvesting and idx ~= CraftingLib.CodeEnumTradeskill.Runecrafting then
			table.insert(tTradeskillsList, { tCurrTradeskill, tCurrTradeskillInfo })
		end
	end
end

function TradeskillTracker:BuildKnownSkillBook(tKnownRecipes)
	local tKnownRecipes = tKnownRecipes
	if tKnownRecipes ~= nil then
		tSkillBook = { }
		
		for idx, tRecipes in pairs(tKnownRecipes) do
			local strRecipeName = tKnownRecipes[idx][1].strName
			local tMatsList = tKnownRecipes[idx][2].tMaterials
			local tComponents = { }
			local strTradeskillName= CraftingLib.GetTradeskillInfo(tKnownRecipes[idx][2].eTradeskillId).strName
			local bTradeskillActive = CraftingLib.GetTradeskillInfo(nTradeId).bIsActive
			local nItemQuality = tKnownRecipes[idx][2].itemOutput:GetItemQuality()
			local strItemCategoryName = tKnownRecipes[idx][2].itemOutput:GetItemCategoryName()
			local strItemTypeName = tKnownRecipes[idx][2].itemOutput:GetItemTypeName()
			local nRequiredLevel = tKnownRecipes[idx][2].itemOutput:GetRequiredLevel()
			local bIsUnique = tKnownRecipes[idx][2].itemOutput:IsUnique()
			local tSubRecipes = tKnownRecipes[idx][2].tSubRecipes
			
			
			for idx2,tMats in ipairs (tMatsList) do
				local strMaterial = tMatsList[idx2].itemMaterial:GetName()
				local nMaterialId = tMatsList[idx2].itemMaterial:GetItemId()
				local strMaterialCategory = tMatsList[idx2].itemMaterial:GetItemTypeName()
				local nMaterialCategory = tMatsList[idx2].itemMaterial:GetItemType()
				local nNumNeeded = tMatsList[idx2].nAmount
				local nSchematicId = tMatsList[idx2].itemMaterial:GetTradeskillSchematicId()
				local bIsCommodity = tMatsList[idx2].itemMaterial:IsCommodity()
				local bIsAuctionable = tMatsList[idx2].itemMaterial:IsAuctionable()
				local nBuyPrice = tMatsList[idx2].itemMaterial:GetBuyPrice():GetAmount()
				local strCurrencyName = tMatsList[idx2].itemMaterial:GetBuyPrice():GetTypeString()
				tComponents[#tComponents + 1]={["Material"] = strMaterial, ["Quantity"] = nNumNeeded, ["BuyPrice"] = nBuyPrice, ["CurrencyType"] = strCurrencyName, ["MaterialID"] = nMaterialId, ["MaterialCategory"] = strMaterialCategory, ["MaterialCategoryId"] = nMaterialCategory, ["IsCommodity"] = bIsCommodity, ["IsAuctionable"] = bIsAuctionable, ["SchematicId"] = nSchematicId}
			end				
			local nRecipeId = tKnownRecipes[idx][1].nSchematicId
			local eSchematicTier = tKnownRecipes[idx][2].eTier
			tSkillBook[#tSkillBook + 1] = {["TradeskillName"] = strTradeskillName, ["TradeskillActive"] = bTradeskillActive, ["RecipeName"] = strRecipeName, ["Quality"] = nItemQuality, ["RequiredLevel"] = nRequiredLevel, ["RecipeTier"] = eSchematicTier, ["CategoryName"] = strItemCategoryName, ["Type"] = strItemTypeName, ["Materials"] = tComponents, ["RecipeId"] = nRecipeId, ["RequiredRecipes"] = tSubRecipes, ["IsUnique"] = bIsUnique}			
		end
	end
	return tSkillBook
end

function TradeskillTracker:BuildKnownTradeskills(tTradeskills)
	tTradesKnown = tTradeskills
	tKnownTrades = { }
	if tTradesKnown ~= nil then
		for idx, tTrades in ipairs(tTradesKnown) do
			strTradeName = tTradesKnown[idx][1].strName
			nTradeTier = tTradesKnown[idx][2].eTier
			nTradeId = tTradesKnown[idx][1].eId
			tSkillLevel = {"Novice ","Apprentice ","Journeyman ","Artisan ","Expert ","Expert Research "}
			strSkillLevel = tSkillLevel[nTradeTier]
			bActive = CraftingLib.GetTradeskillInfo(nTradeId).bIsActive
			if strTradeName and strSkillLevel then
				tKnownTrades[#tKnownTrades+1] = {["TradeskillName"] = strTradeName,["TradeskillLevel"] = strSkillLevel,["TradeskillTitle"] = strSkillLevel..strTradeName, ["TradeId"] = nTradeId, ["Active"] = bActive}
			else
			end
		end
	end
	return tKnownTrades
end	

function TradeskillTracker:BuildTradeskillMetaData(nKnown)
	local tTrades = tTradeskillChapter
	local tRecipes = tRecipesChapter
	tMetaData = { }
	local strRealm = GameLib.GetRealmName()
	local strUnit = GameLib.GetPlayerUnit()
	local strPlayerName = strUnit:GetName()
	local strFaction = strUnit:GetFaction()
	local nTime = os.time()
	local nKnownRecipes = nKnown
	local strProfileId = ""
	if self.wndMain:FindChild("strProfileId"):GetText() ~= nil then
		strProfileId = self.wndMain:FindChild("strProfileId"):GetText()
	else
		strProfileId = self:GenerateKey()
		self.wndMain:FindChild("strProfileId"):SetText(strProfileId)
	end
	
	tMetaData = {["ServerName"] = strRealm, ["Faction"] = strFaction, ["PlayerName"] = strPlayerName, ["ProfileId"] = strProfileId, ["LastExported"] = nTime, ["KnownRecipes"] = nKnownRecipes}

	return tMetaData

end

function TradeskillTracker:BuildSortedTradeskillContent(tTradeskillChapter,tRecipeChapter)
	local tTrades = tTradeskillChapter
	local tRecipes = tRecipeChapter
	local tSortedRecipesByTrade = { }
	for idx, trade in pairs (tTrades) do
		tSortedRecipesByTrade[#tSortedRecipesByTrade + 1] = {["TradeName"] = tTrades[idx].TradeskillName, ["TradeskillDetails"] = {["TradeskillTitle"] = tTrades[idx].TradeskillTitle, ["TradeskillLevel"] = tTrades[idx].TradeskillLevel,["TradeId"] = tTrades[idx].TradeId, ["Active"] = tTrades[idx].Active},["TradeskillRecipes"] = {}}
		for idx2, recipe in pairs (tRecipes) do
			if tTrades[idx].TradeskillName == tRecipes[idx2].TradeskillName then
				table.insert(tSortedRecipesByTrade[idx].TradeskillRecipes,{["Name"] = tRecipes[idx2].RecipeName, ["RecipeDetails"] = {["Quality"] = tRecipes[idx2].Quality, ["RequiredLevel"] = tRecipes[idx2].RequiredLevel, ["RecipeTier"] = tRecipes[idx2].RecipeTier, ["CategoryName"] = tRecipes[idx2].CategoryName, ["Type"] = tRecipes[idx2].Type, ["Materials"] = tRecipes[idx2].Materials, ["RecipeId"] = tRecipes[idx2].RecipeId, ["RequiredRecipes"] = tRecipes[idx2].RequiredRecipes, ["IsUnique"] = tRecipes[idx2].IsUnique}})
			end
		end
	end
	return tSortedRecipesByTrade

end


-----------------------------------------------------------------------------------------------
-- TradeskillTrackerForm Functions
-----------------------------------------------------------------------------------------------

-- when the Cancel button is clicked
function TradeskillTracker:OnCancel()
	self.wndMain:Close() -- hide the window
end

function TradeskillTracker:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "Tradeskill Tracker"})
end


-----------------------------------------------------------------------------------------------
-- ItemList Functions
-----------------------------------------------------------------------------------------------
-- populate item list
function TradeskillTracker:PopulateItemList(tTradeskillChapter,tRecipeChapter)
	-- make sure the item list is empty to start with
	self:DestroyItemList()
	
	tActiveTradeskills = { }
	for idx, field in pairs(tTradeskillChapter) do
		if tTradeskillChapter[idx].Active == true then
			local strTradeskillTitle = tTradeskillChapter[idx].TradeskillTitle
			local nTradeId = tTradeskillChapter[idx].TradeId
			local strTradeskillName = tTradeskillChapter[idx].TradeskillName
			tRecipesByTradeskill = { }
			for idx2, recipe in pairs(tRecipeChapter) do
				if tRecipeChapter[idx2].TradeskillName == strTradeskillName then
					local strRecipeName = tRecipeChapter[idx2].RecipeName
					local nRecipeId = tRecipeChapter[idx2].RecipeId
					tRecipesByTradeskill[#tRecipesByTradeskill + 1] = {["Recipe Name"] = strRecipeName}
				end
			end
			tActiveTradeskills[#tActiveTradeskills + 1] = {["TradeskillName"] = tTradeskillChapter[idx].TradeskillTitle, {["Recipes"] = tRecipesByTradeskill}, ["nTradeskillRecipes"] = #tRecipesByTradeskill}
		end
	end
    -- add each Tradeskill
	for i = 1,#tActiveTradeskills do
		if tActiveTradeskills[i].nTradeskillRecipes > 0 then
        	self:AddItem(tActiveTradeskills[i].TradeskillName..": "..tActiveTradeskills[i].nTradeskillRecipes.." recipes known")
		end
	end
	
	-- now all the item are added, call ArrangeChildrenVert to list out the list items vertically
	self.wndItemList:ArrangeChildrenVert()
end

-- clear the item list
function TradeskillTracker:DestroyItemList()
	-- destroy all the wnd inside the list
	for idx,wnd in ipairs(self.tItems) do
		wnd:Destroy()
	end

	-- clear the list item array
	self.tItems = {}
	self.wndSelectedListItem = nil
end

-- add an item into the item list
function TradeskillTracker:AddItem(i)
	-- load the window item for the list item
	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)
	
	-- keep track of the window item created
	self.tItems[i] = wnd

	-- give it a piece of data to refer to 
	local wndItemText = wnd:FindChild("Text")
	if wndItemText then -- make sure the text wnd exist
		wndItemText:SetText(i) -- set the item wnd's text to Tradeskill Name
		wndItemText:SetTextColor(kcrNormalText)
	end
	wnd:SetData(i)
end

-- when a list item is selected
function TradeskillTracker:OnListItemSelected(wndHandler, wndControl)
    -- make sure the wndControl is valid
    if wndHandler ~= wndControl then
        return
    end
    
    -- change the old item's text color back to normal color
    local wndItemText
    if self.wndSelectedListItem ~= nil then
        wndItemText = self.wndSelectedListItem:FindChild("Text")
        wndItemText:SetTextColor(kcrNormalText)
    end
    
	-- wndControl is the item selected - change its color to selected
	self.wndSelectedListItem = wndControl
	wndItemText = self.wndSelectedListItem:FindChild("Text")
    wndItemText:SetTextColor(kcrSelectedText)
    --Not necessary to print what people can already read.  Possible TODO to breakout each recipe name, but this is just
	--a summary report of what is being submitted to tradeskill tracking database on save.
	--Print(self.wndSelectedListItem:GetData() .. " is selected.")
end

----------------------------------------------------------------------------------------------
-- Helpers
----------------------------------------------------------------------------------------------
function TradeskillTracker:GenerateKey()
	math.randomseed(math.random(1, 100000000))
    -- Reseed the generator multiple times based on length of character name
	unit = GameLib.GetPlayerUnit():GetName()
	nUnitNameLength = #unit
	for index = 1, nUnitNameLength do
    	math.randomseed(math.random(1,100000000))
	end
	
       
    local characters = {'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
                        'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
                        'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D',
						'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
						'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
						'Y', 'Z', '0', '1', '2', '3', '4', '5', '6', '7',
						'8', '9', '!', '@', '#', '$', '^'
                           }
    local key = ""
    for index = 1, 10 do
    	key = key .. characters[math.random(1, 67)]
		math.randomseed(math.random(1,100000000))
    end
       
    return key
end

-----------------------------------------------------------------------------------------------
-- TradeskillTracker Instance
-----------------------------------------------------------------------------------------------
local TradeskillTrackerInst = TradeskillTracker:new()
TradeskillTrackerInst:Init()
